@@ -14,6 +14,9 @@
 
 #include "ui_fileops.h"
 
+#include <utime.h>
+#include <errno.h>
+
 
 /*
  *-------------------------------------------------------------------
@@ -541,3 +544,43 @@ gchar *cache_find_location(const gchar *source, const gchar *ext)
 	return path;
 }
 
+gint cache_time_valid(const gchar *cache, const gchar *path)
+{
+	struct stat cache_st;
+	struct stat path_st;
+	gchar *cachel;
+	gchar *pathl;
+	gint ret = FALSE;
+
+	if (!cache || !path) return FALSE;
+
+	cachel = path_from_utf8(cache);
+	pathl = path_from_utf8(path);
+
+	if (stat(cachel, &cache_st) == 0 &&
+	    stat(pathl, &path_st) == 0)
+		{
+		if (cache_st.st_mtime == path_st.st_mtime)
+			{
+			ret = TRUE;
+			}
+		else if (cache_st.st_mtime > path_st.st_mtime)
+			{
+			struct utimbuf ut;
+
+			ut.actime = ut.modtime = cache_st.st_mtime;
+			if (utime(cachel, &ut) < 0 &&
+			    errno == EPERM)
+				{
+				if (debug) printf("cache permission workaround: %s\n", cachel);
+				ret = TRUE;
+				}
+			}
+		}
+
+	g_free(pathl);
+	g_free(cachel);
+
+	return ret;
+}
+
