@@ -855,7 +855,7 @@ static void collection_table_set_focus(CollectTable *ct, CollectInfo *info)
 		GtkTreeViewColumn *column;
 		GtkTreeModel *store;
 
-		tree_view_row_make_visible(GTK_TREE_VIEW(ct->listview), &iter);
+		tree_view_row_make_visible(GTK_TREE_VIEW(ct->listview), &iter, FALSE);
 
 		store = gtk_tree_view_get_model(GTK_TREE_VIEW(ct->listview));
 		tpath = gtk_tree_model_get_path(store, &iter);
@@ -1295,8 +1295,8 @@ static void collection_table_insert_marker(CollectTable *ct, CollectInfo *info,
 		gdk_window_set_back_pixmap(ct->marker_window, pixmap, FALSE);
 		gdk_window_shape_combine_mask(ct->marker_window, mask, 0, 0);
 
-		gdk_pixmap_unref(pixmap);
-		if (mask) gdk_bitmap_unref(mask);
+		g_object_unref(pixmap);
+		if (mask) g_object_unref(mask);
 		}
 
 	if (info)
@@ -1361,7 +1361,7 @@ static gint collection_table_auto_scroll_idle_cb(gpointer data)
 
 	window = ct->listview->window;
 	gdk_window_get_pointer(window, &x, &y, NULL);
-	gdk_window_get_size(window, &w, &h);
+	gdk_drawable_get_size(window, &w, &h);
 	if (x >= 0 && x < w && y >= 0 && y < h)
 		{
 		collection_table_motion_update(ct, x, y, TRUE);
@@ -1877,6 +1877,11 @@ void collection_table_file_insert(CollectTable *ct, CollectInfo *ci)
 
 void collection_table_file_remove(CollectTable *ct, CollectInfo *ci)
 {
+	if (ci && INFO_SELECTED(ci))
+		{
+		ct->selection = g_list_remove(ct->selection, ci);
+		}
+
 	collection_table_sync_idle(ct);
 }
 
