@@ -67,7 +67,7 @@ static GList *dupe_window_list = NULL;	/* list of open DupeWindow *s */
 static void dupe_match_unlink(DupeItem *a, DupeItem *b);
 static DupeItem *dupe_match_find_parent(DupeWindow *dw, DupeItem *child);
 
-static gint dupe_match(DupeItem *a, DupeItem *b, DupeMatchType mask, gfloat *rank);
+static gint dupe_match(DupeItem *a, DupeItem *b, DupeMatchType mask, gfloat *rank, gint fast);
 
 static void dupe_thumb_step(DupeWindow *dw);
 static gint dupe_check_cb(gpointer data);
@@ -568,6 +568,8 @@ static void dupe_listview_remove(DupeWindow *dw, DupeItem *di)
 	store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(dw->listview)));
 	row = dupe_listview_find_item(store, di, &iter);
 	if (row < 0) return;
+
+	tree_view_move_cursor_away(GTK_TREE_VIEW(dw->listview), &iter, TRUE);
 	gtk_list_store_remove(store, &iter);
 
 	if (g_list_find(dw->dupes, di) != NULL)
@@ -683,7 +685,8 @@ static DupeMatch *dupe_match_find_match(DupeItem *child, DupeItem *parent)
 static void dupe_match_link_child(DupeItem *child, DupeItem *parent, gfloat rank)
 {
 	DupeMatch *dm;
-	                                                                                                                               dm = g_new0(DupeMatch, 1);
+
+	dm = g_new0(DupeMatch, 1);
 	dm->di = child;
 	dm->rank = rank;
 	parent->group = g_list_append(parent->group, dm);
@@ -898,7 +901,7 @@ static GList *dupe_match_unlink_by_rank(DupeItem *child, DupeItem *parent, GList
 				dupe_match_link_clear(orphan, TRUE);
 				if (!dw->second_set || orphan->second)
 					{
-					dupe_match(orphan, child, dw->match_mask, &rank);
+					dupe_match(orphan, child, dw->match_mask, &rank, FALSE);
 					dupe_match_link(orphan, child, rank);
 					}
 				list = g_list_remove(list, orphan);
@@ -1037,87 +1040,39 @@ static void dupe_match_rank(DupeWindow *dw)
  * ------------------------------------------------------------------
  */
 
-static gint dupe_match(DupeItem *a, DupeItem *b, DupeMatchType mask, gfloat *rank)
+static gint dupe_match(DupeItem *a, DupeItem *b, DupeMatchType mask, gfloat *rank, gint fast)
 {
-	gint match = 0;
+	*rank = 0.0;
 
 	if (a == b) return FALSE;
 
 	if (mask & DUPE_MATCH_PATH)
 		{
-		if (strcmp(a->path, b->path) == 0)
-			{
-			match = 1;
-			*rank = 0;
-			}
-		else
-			{
-			return FALSE;
-			}
+		if (strcmp(a->path, b->path) != 0) return FALSE;
 		}
 	if (mask & DUPE_MATCH_NAME)
 		{
-		if (strcmp(a->name, b->name) == 0)
-			{
-			match = 1;
-			*rank = 0;
-			}
-		else
-			{
-			return FALSE;
-			}
+		if (strcmp(a->name, b->name) != 0) return FALSE;
 		}
 	if (mask & DUPE_MATCH_SIZE)
 		{
-		if (a->size == b->size)
-			{
-			match = 1;
-			*rank = 0;
-			}
-		else
-			{
-			return FALSE;
-			}
+		if (a->size != b->size) return FALSE;
 		}
 	if (mask & DUPE_MATCH_DATE)
 		{
-		if (a->date == b->date)
-			{
-			match = 1;
-			*rank = 0;
-			}
-		else
-			{
-			return FALSE;
-			}
+		if (a->date != b->date) return FALSE;
 		}
 	if (mask & DUPE_MATCH_SUM)
 		{
 		if (a->checksum == 0) a->checksum = checksum_simple(a->path);
 		if (b->checksum == 0) b->checksum = checksum_simple(b->path);
-		if (a->checksum == b->checksum)
-			{
-			match = 1;
-			*rank = 0;
-			}
-		else
-			{
-			return FALSE;
-			}
+		if (a->checksum != b->checksum) return FALSE;
 		}
 	if (mask & DUPE_MATCH_DIM)
 		{
 		if (a->width == 0) image_load_dimensions(a->path, &a->width, &a->height);
 		if (b->width == 0) image_load_dimensions(b->path, &b->width, &b->height);
-		if (a->width == b->width && a->height == b->height)
-			{
-			match = 1;
-			*rank = 0;
-			}
-		else
-			{
-			return FALSE;
-			}
+		if (a->width != b->width || a->height != b->height) return FALSE;
 		}
 	if (mask & DUPE_MATCH_SIM_HIGH ||
 	    mask & DUPE_MATCH_SIM_MED ||
@@ -1132,21 +1087,23 @@ static gint dupe_match(DupeItem *a, DupeItem *b, DupeMatchType mask, gfloat *ran
 		else if (mask & DUPE_MATCH_SIM_CUSTOM) m = (float)dupe_custom_threshold / 100.0;
 		else m = 0.85;
 
-		f = image_sim_compare_fast(a->simd, b->simd, m);
-
-		if (f > m)
+		if (fast)
 			{
-			if (debug > 2) printf("similar: %32s %32s = %f\n", a->name, b->name, f);
-			match = 1;
-			*rank = f * 100.0;
+			f = image_sim_compare_fast(a->simd, b->simd, m);
 			}
 		else
 			{
-			return FALSE;
+			f = image_sim_compare(a->simd, b->simd);
 			}
+
+		*rank = f * 100.0;
+
+		if (f < m) return FALSE;
+
+		if (debug > 2) printf("similar: %32s %32s = %f\n", a->name, b->name, f);
 		}
 
-	return match;
+	return TRUE;
 }
 
 static void dupe_list_check_match(DupeWindow *dw, DupeItem *needle, GList *start)
@@ -1180,7 +1137,7 @@ static void dupe_list_check_match(DupeWindow *dw, DupeItem *needle, GList *start
 			{
 			gfloat rank;
 
-			if (dupe_match(di, needle, dw->match_mask, &rank))
+			if (dupe_match(di, needle, dw->match_mask, &rank, TRUE))
 				{
 				dupe_match_link(di, needle, rank);
 				}
@@ -2202,16 +2159,16 @@ static GtkWidget *dupe_menu_popup_main(DupeWindow *dw, DupeItem *di)
 static gint dupe_listview_press_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer data)
 {
 	DupeWindow *dw = data;
+	GtkTreeModel *store;
 	GtkTreePath *tpath;
 	GtkTreeIter iter;
 	DupeItem *di = NULL;
 
+	store = gtk_tree_view_get_model(GTK_TREE_VIEW(widget));
+
 	if (gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(widget), bevent->x, bevent->y,
 					  &tpath, NULL, NULL, NULL))
 		{
-		GtkTreeModel *store;
-
-		store = gtk_tree_view_get_model(GTK_TREE_VIEW(widget));
 		gtk_tree_model_get_iter(store, &iter, tpath);
 		gtk_tree_model_get(store, &iter, DUPE_COLUMN_POINTER, &di, -1);
 		gtk_tree_path_free(tpath);
@@ -2258,8 +2215,12 @@ static gint dupe_listview_press_cb(GtkWidget *widget, GdkEventButton *bevent, gp
 			selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(widget));
 			gtk_tree_selection_unselect_all(selection);
 			gtk_tree_selection_select_iter(selection, &iter);
+
+			tpath = gtk_tree_model_get_path(GTK_TREE_MODEL(store), &iter);
+			gtk_tree_view_set_cursor(GTK_TREE_VIEW(widget), tpath, NULL, FALSE);
+			gtk_tree_path_free(tpath);
 			}
-		/* do not allow the treeview to change the selection */
+
 		return TRUE;
 		}
 
@@ -2269,6 +2230,7 @@ static gint dupe_listview_press_cb(GtkWidget *widget, GdkEventButton *bevent, gp
 	    dupe_listview_item_is_selected(dw, di, widget))
 		{
 		/* this selection handled on release_cb */
+		gtk_widget_grab_focus(widget);
 		return TRUE;
 		}
 
@@ -2278,19 +2240,19 @@ static gint dupe_listview_press_cb(GtkWidget *widget, GdkEventButton *bevent, gp
 static gint dupe_listview_release_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer data)
 {
 	DupeWindow *dw = data;
+	GtkTreeModel *store;
 	GtkTreePath *tpath;
 	GtkTreeIter iter;
 	DupeItem *di = NULL;
 
 	if (bevent->button != 1 && bevent->button != 2) return TRUE;
 
+	store = gtk_tree_view_get_model(GTK_TREE_VIEW(widget));
+
 	if ((bevent->x != 0 || bevent->y != 0) &&
 	    gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(widget), bevent->x, bevent->y,
 					  &tpath, NULL, NULL, NULL))
 		{
-		GtkTreeModel *store;
-
-		store = gtk_tree_view_get_model(GTK_TREE_VIEW(widget));
 		gtk_tree_model_get_iter(store, &iter, tpath);
 		gtk_tree_model_get(store, &iter, DUPE_COLUMN_POINTER, &di, -1);
 		gtk_tree_path_free(tpath);
@@ -2325,6 +2287,11 @@ static gint dupe_listview_release_cb(GtkWidget *widget, GdkEventButton *bevent,
 		selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(widget));
 		gtk_tree_selection_unselect_all(selection);
 		gtk_tree_selection_select_iter(selection, &iter);
+
+		tpath = gtk_tree_model_get_path(store, &iter);
+		gtk_tree_view_set_cursor(GTK_TREE_VIEW(widget), tpath, NULL, FALSE);
+		gtk_tree_path_free(tpath);
+
 		return TRUE;
 		}
 
@@ -2371,6 +2338,7 @@ static void dupe_second_remove(DupeWindow *dw, DupeItem *di)
 	store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(dw->second_listview)));
 	if (dupe_listview_find_item(store, di, &iter) >= 0)
 		{
+		tree_view_move_cursor_away(GTK_TREE_VIEW(dw->second_listview), &iter, TRUE);
 		gtk_list_store_remove(store, &iter);
 		}
 
@@ -3276,7 +3244,7 @@ static GtkWidget *dupe_confirm_dir_list(DupeWindow *dw, GList *list)
 	GtkWidget *menu;
 	CDupeConfirmD *d;
 
-	d = g_new(CDupeConfirmD, 1);
+	d = g_new0(CDupeConfirmD, 1);
 	d->dw = dw;
 	d->list = list;
 
@@ -3419,10 +3387,15 @@ static void dupe_dnd_begin(GtkWidget *widget, GdkDragContext *context, gpointer
 		if (dupe_listview_find_item(store, dw->click_item, &iter) >= 0)
 			{
 			GtkTreeSelection *selection;
+			GtkTreePath *tpath;
 
 			selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(widget));
 			gtk_tree_selection_unselect_all(selection);
 			gtk_tree_selection_select_iter(selection, &iter);
+
+			tpath = gtk_tree_model_get_path(GTK_TREE_MODEL(store), &iter);
+			gtk_tree_view_set_cursor(GTK_TREE_VIEW(widget), tpath, NULL, FALSE);
+			gtk_tree_path_free(tpath);
 			}
 		}
 
