@@ -235,6 +235,8 @@ static gint editor_verbose_start(EditorVerboseData *vd, const gchar *command)
 	{
 	FILE *f;
 	GIOChannel *channel;
+	int fd;
+	int flags;
 
 	f = popen(command, "r");
 	if (!f)
@@ -248,6 +250,17 @@ static gint editor_verbose_start(EditorVerboseData *vd, const gchar *command)
 		return FALSE;
 		}
 
+	/* set our end of pipe non blocking */
+	fd = fileno(f);
+	if ((flags = fcntl(fd, F_GETFL)) >= 0)
+		{
+		flags |= O_NONBLOCK;
+		if (fcntl(fd, F_SETFL, &flags) < 0)
+			{
+			printf("Unable to set non blocking mode for editor pipe\n");
+			}
+		}
+
 	vd->fp = f;
 	channel = g_io_channel_unix_new(fileno(vd->fp));
 	g_io_add_watch_full(channel, G_PRIORITY_DEFAULT, G_IO_IN,
