@@ -45,7 +45,7 @@ static void clear_mouse_cursor(GtkWidget *widget, gint clear)
 		gdk_window_set_cursor (widget->window, cursor);
 
 		gdk_cursor_unref(cursor);
-		gdk_pixmap_unref(p);
+		g_object_unref(p);
 		}
 	else
 		{
