@@ -1,6 +1,6 @@
 /*
  * GQview
- * (C) 2001 John Ellis
+ * (C) 2004 John Ellis
  *
  * Author: John Ellis
  *
@@ -17,22 +17,6 @@
 #define BLACK_BACKGROUND 1
 
 
-typedef struct _FullScreenData FullScreenData;
-struct _FullScreenData
-{
-	GtkWidget *window;
-	ImageWindow *imd;
-
-	GtkWidget *normal_window;
-	ImageWindow *normal_imd;
-
-	gint hide_mouse_id;
-
-	void (*stop_func)(FullScreenData *, gpointer);
-	gpointer stop_data;
-};
-
-
 FullScreenData *fullscreen_start(GtkWidget *window, ImageWindow *imd,
 				 void (*stop_func)(FullScreenData *, gpointer), gpointer stop_data);
 void fullscreen_stop(FullScreenData *fs);
@@ -40,3 +24,4 @@ void fullscreen_stop(FullScreenData *fs);
 
 #endif
 
+
