@@ -1,6 +1,6 @@
 /*
  * GQview
- * (C) 2002 John Ellis
+ * (C) 2004 John Ellis
  *
  * Author: John Ellis
  *
@@ -18,13 +18,13 @@
 GList *filename_filter = NULL;
 
 /* -- options -- */
-gint main_window_w = 400;
-gint main_window_h = 350;
+gint main_window_w = 500;
+gint main_window_h = 400;
 gint main_window_x = 0;
 gint main_window_y = 0;
 
-gint float_window_w = 150;
-gint float_window_h = 350;
+gint float_window_w = 260;
+gint float_window_h = 450;
 gint float_window_x = 0;
 gint float_window_y = 0;
 gint float_window_divider = -1;
@@ -42,6 +42,9 @@ gint startup_path_enable = FALSE;
 gchar *startup_path = NULL;
 gint confirm_delete = TRUE;
 gint enable_delete_key = TRUE;
+gint safe_delete_enable = FALSE;
+gchar *safe_delete_path = NULL;
+gint safe_delete_size = 20;
 gint restore_tool = FALSE;
 gint zoom_mode = ZOOM_RESET_ORIGINAL;
 gint two_pass_zoom = TRUE;
@@ -51,23 +54,13 @@ gint zoom_to_fit_expands = TRUE;
 gint max_window_size = 100;
 gint thumb_max_width = 64;
 gint thumb_max_height = 64;
-gint enable_thumb_caching = FALSE;
+gint enable_thumb_caching = TRUE;
 gint enable_thumb_dirs = FALSE;
 gint use_xvpics_thumbnails = TRUE;
 gint show_dot_files = FALSE;
 gint file_filter_disable = FALSE;
-gint filter_include_jpg = TRUE;
-gint filter_include_xpm = TRUE;
-gint filter_include_tif = TRUE;
-gint filter_include_gif = TRUE;
-gint filter_include_png = TRUE;
-gint filter_include_ppm = TRUE;
-gint filter_include_pgm = TRUE;
-gint filter_include_pcx = TRUE;
-gint filter_include_bmp = TRUE;
-gchar *custom_filter = NULL;
-gchar *editor_name[8];
-gchar *editor_command[8];
+gchar *editor_name[GQVIEW_EDITOR_SLOTS];
+gchar *editor_command[GQVIEW_EDITOR_SLOTS];
 
 gint thumbnails_enabled = FALSE;
 SortType file_sort_method = SORT_NAME;
@@ -83,7 +76,6 @@ gint enable_in_place_rename = TRUE;
 gint recent_list_max = 10;
 
 gint collection_rectangular_selection = FALSE;
-gint collection_drop_insert = TRUE;
 
 gint tile_cache_max = 10;
 gint thumbnail_quality = (gint)GDK_INTERP_NEAREST;
@@ -94,7 +86,7 @@ gint zoom_increment = 5;
 
 gint enable_read_ahead = TRUE;
 
-gint place_dialogs_under_mouse = TRUE;
+gint place_dialogs_under_mouse = FALSE;
 
 gint black_window_background = FALSE;
 
@@ -114,6 +106,11 @@ gint layout_style = 0;
 gint layout_view_icons = FALSE;
 gint layout_view_tree = FALSE;
 
+gint show_icon_names = TRUE;
+
 gint tree_descend_subdirs = FALSE;
 
 gint lazy_image_sync = FALSE;
+gint update_on_time_change = TRUE;
+gint exif_rotate_enable = FALSE;
+
