@@ -21,7 +21,6 @@
 #include "ui_fileops.h"
 
 #include <math.h>
-#include <gdk/gdkx.h>
 
 
 #define IMAGE_TILE_SIZE 128
@@ -112,7 +111,7 @@ static void image_update_title(ImageWindow *imd);
 static void image_update_util(ImageWindow *imd);
 static void image_complete_util(ImageWindow *imd, gint preload);
 
-static void image_button_do(ImageWindow *imd, GdkEvent *event, gint button);
+static void image_button_do(ImageWindow *imd, GdkEventButton *bevent);
 
 
 /*
@@ -154,7 +153,7 @@ static void image_tile_cache_free(ImageWindow *imd, CacheData *cd)
 	imd->tile_cache = g_list_remove(imd->tile_cache, cd);
 	if (cd->pixmap)
 		{
-		gdk_pixmap_unref(cd->it->pixmap);
+		g_object_unref(cd->it->pixmap);
 		cd->it->pixmap = NULL;
 		cd->it->render_done = TILE_RENDER_NONE;
 		}
@@ -257,7 +256,7 @@ static void image_tile_free(ImageTile *it)
 	if (!it) return;
 
 	if (it->pixbuf) gdk_pixbuf_unref(it->pixbuf);
-	if (it->pixmap) gdk_pixmap_unref(it->pixmap);
+	if (it->pixmap) g_object_unref(it->pixmap);
 
 	g_free(it);
 }
@@ -414,7 +413,6 @@ static void image_tile_render(ImageWindow *imd, ImageTile *it,
 	else
 		{
 		double scale_x, scale_y;
-		gint simple;
 
 		if (imd->image_width == 0 || imd->image_height == 0) return;
 		scale_x = (double)imd->width / imd->image_width;
@@ -425,16 +423,7 @@ static void image_tile_render(ImageWindow *imd, ImageTile *it,
 		 */
 		if (imd->width < IMAGE_MIN_SCALE_SIZE || imd->height < IMAGE_MIN_SCALE_SIZE) fast = TRUE;
 
-		if (GDK_PIXBUF_TILES_BROKEN)
-			{
-			simple = (!has_alpha && (GdkInterpType)zoom_quality == GDK_INTERP_NEAREST);
-			}
-		else
-			{
-			simple = (!has_alpha);
-			}
-
-		if (simple)
+		if (!has_alpha)
 			{
 			gdk_pixbuf_scale(imd->pixbuf, it->pixbuf, x, y, w, h,
 					 (double) 0.0 - it->x,
@@ -472,9 +461,9 @@ static void image_tile_expose(ImageWindow *imd, ImageTile *it,
 {
 	image_tile_render(imd, it, x, y, w, h, new_data, fast);
 
-	gdk_draw_pixmap (imd->image->window, imd->image->style->fg_gc[GTK_WIDGET_STATE(imd->image)],
-			 it->pixmap, x, y,
-			 imd->x_offset + (it->x - imd->x_scroll) + x, imd->y_offset + (it->y - imd->y_scroll) + y, w, h);
+	gdk_draw_drawable(imd->image->window, imd->image->style->fg_gc[GTK_WIDGET_STATE(imd->image)],
+			  it->pixmap, x, y,
+			  imd->x_offset + (it->x - imd->x_scroll) + x, imd->y_offset + (it->y - imd->y_scroll) + y, w, h);
 }
 
 static gint image_tile_is_visible(ImageWindow *imd, ImageTile *it)
@@ -800,7 +789,7 @@ static gint image_size_top_window(ImageWindow *imd, gint w, gint h)
 	w += (imd->top_window->allocation.width - imd->eventbox->allocation.width);
 	h += (imd->top_window->allocation.height - imd->eventbox->allocation.height);
 
-	gdk_window_get_size(imd->top_window->window, &ww, &wh);
+	gdk_drawable_get_size(imd->top_window->window, &ww, &wh);
 	if (w == ww && h == wh) return FALSE;
 
 	gdk_window_resize(imd->top_window->window, w, h);
@@ -1147,11 +1136,11 @@ static void image_scroll_real(ImageWindow *imd, gint x, gint y)
 
 		gc = gdk_gc_new(imd->image->window);
 		gdk_gc_set_exposures(gc, TRUE);
-		gdk_window_copy_area(imd->image->window, gc,
-				     x1 + imd->x_offset, y1 + imd->y_offset,
-				     imd->image->window,
-				     x2 + imd->x_offset, y2 + imd->y_offset, w, h);
-		gdk_gc_destroy(gc);
+		gdk_draw_drawable(imd->image->window, gc,
+				  imd->image->window,
+				  x2 + imd->x_offset, y2 + imd->y_offset,
+				  x1 + imd->x_offset, y1 + imd->y_offset, w, h);
+		g_object_unref(gc);
 
 		/*
 		 * RACE CONDITION avoidance for expose_event and scrolling (hack):
@@ -2023,7 +2012,7 @@ static gint image_mouse_press_cb(GtkWidget *widget, GdkEventButton *bevent, gpoi
 			imd->drag_moved = 0;
 			break;
 		case 3:
-			image_button_do(imd, (GdkEvent *)bevent, bevent->button);
+			image_button_do(imd, bevent);
 			break;
 		default:
 			break;
@@ -2047,7 +2036,7 @@ static gint image_mouse_release_cb(GtkWidget *widget, GdkEventButton *bevent, gp
 
 	if (bevent->button == 1 || bevent->button == 2)
 		{
-		if (imd->drag_moved < IMAGE_DRAG_SCROLL_THRESHHOLD) image_button_do(imd, (GdkEvent *)bevent, bevent->button);
+		if (imd->drag_moved < IMAGE_DRAG_SCROLL_THRESHHOLD) image_button_do(imd, bevent);
 		}
 
 	imd->in_drag = FALSE;
@@ -2059,14 +2048,11 @@ static gint image_scroll_cb(GtkWidget *widget, GdkEventScroll *event, gpointer d
 {
 	ImageWindow *imd = data;
 
-	if (event->direction == GDK_SCROLL_UP)
-		{
-		image_button_do(imd, (GdkEvent *)event, 4);
-		return TRUE;
-		}
-	else if (event->direction == GDK_SCROLL_DOWN)
-		{
-		image_button_do(imd, (GdkEvent *)event, 5);
+	if (imd->func_scroll &&
+	    event && event->type == GDK_SCROLL)
+	        {
+		imd->func_scroll(imd, event->direction, event->time,
+				 event->x, event->y, event->state, imd->data_scroll);
 		return TRUE;
 		}
 
@@ -2122,81 +2108,31 @@ void image_set_complete_func(ImageWindow *imd,
 }
 
 
-static void image_button_do(ImageWindow *imd, GdkEvent *event, gint button)
+static void image_button_do(ImageWindow *imd, GdkEventButton *bevent)
 {
-	void (*func)(ImageWindow *, guint32 time, gdouble x, gdouble y, guint state, gpointer);
-	gpointer data;
-
-	switch (button)
+	if (imd->func_button &&
+	    bevent &&
+	    (bevent->type == GDK_BUTTON_PRESS || bevent->type == GDK_BUTTON_RELEASE))
 		{
-		case 1:
-			func = imd->func_btn1;
-			data = imd->data_btn1;
-			break;
-		case 2:
-			func = imd->func_btn2;
-			data = imd->data_btn2;
-			break;
-		case 3:
-			func = imd->func_btn3;
-			data = imd->data_btn3;
-			break;
-		case 4:
-			func = imd->func_btn4;
-			data = imd->data_btn4;
-			break;
-		case 5:
-			func = imd->func_btn5;
-			data = imd->data_btn5;
-			break;
-		default:
-			func = NULL;
-			data = NULL;
-			break;
-		}
-
-	if (func)
-		{
-		if (event->type == GDK_BUTTON_PRESS || event->type == GDK_BUTTON_RELEASE)
-			{
-			GdkEventButton *bevent = (GdkEventButton *)event;
-			func(imd, bevent->time, bevent->x, bevent->y, bevent->state, data);
-			}
-		else if (event->type == GDK_SCROLL)
-			{
-			GdkEventScroll *sevent = (GdkEventScroll *)event;
-			func(imd, sevent->time, sevent->x, sevent->y, sevent->state, data);
-			}
+		imd->func_button(imd, bevent->button, bevent->time,
+				 bevent->x, bevent->y, bevent->state, imd->data_button);
 		}
 }
 
-void image_set_button_func(ImageWindow *imd, gint button,
-			   void (*func)(ImageWindow *, guint32 time, gdouble x, gdouble y, guint state, gpointer),
+void image_set_button_func(ImageWindow *imd,
+			   void (*func)(ImageWindow *, gint button, guint32 time, gdouble x, gdouble y, guint state, gpointer),
 			   gpointer data)
 {
-	switch (button)
-		{
-		case 1:
-			imd->func_btn1 = func;
-			imd->data_btn1 = data;
-			break;
-		case 2:
-			imd->func_btn2 = func;
-			imd->data_btn2 = data;
-			break;
-		case 3:
-			imd->func_btn3 = func;
-			imd->data_btn3 = data;
-			break;
-		case 4:
-			imd->func_btn4 = func;
-			imd->data_btn4 = data;
-			break;
-		case 5:
-			imd->func_btn5 = func;
-			imd->data_btn5 = data;
-			break;
-		}
+	imd->func_button = func;
+	imd->data_button = data;
+}
+
+void image_set_scroll_func(ImageWindow *imd,
+			   void (*func)(ImageWindow *, GdkScrollDirection direction, guint32 time, gdouble x, gdouble y, guint state, gpointer),
+			   gpointer data)
+{
+	imd->func_scroll = func;
+	imd->data_scroll = data;
 }
 
 /* path, name */
@@ -2602,7 +2538,7 @@ void image_background_set_color(ImageWindow *imd, GdkColor *color)
 
 void image_to_root_window(ImageWindow *imd, gint scaled)
 {
-	GdkVisual *gdkvisual;
+	GdkScreen *screen;
 	GdkWindow *rootwindow;
 	GdkPixmap *pixmap;
 	GdkPixbuf *pb;
@@ -2610,9 +2546,9 @@ void image_to_root_window(ImageWindow *imd, gint scaled)
 	if (!imd || !imd->pixbuf) return;
 
 
-	rootwindow = GDK_ROOT_PARENT();	/* hmm, don't know, correct? */
-	gdkvisual = gdk_window_get_visual(rootwindow);
-	if (gdkvisual != gdk_visual_get_system()) return;
+	screen = gtk_widget_get_screen(imd->image);
+	rootwindow = gdk_screen_get_root_window(screen);
+	if (gdk_drawable_get_visual(rootwindow) != gdk_visual_get_system()) return;
 
 	if (scaled)
 		{
@@ -2704,11 +2640,8 @@ ImageWindow *image_new(gint frame)
 	imd->func_update = NULL;
 	imd->func_complete = NULL;
 
-	imd->func_btn1 = NULL;
-	imd->func_btn2 = NULL;
-	imd->func_btn3 = NULL;
-	imd->func_btn4 = NULL;
-	imd->func_btn5 = NULL;	
+	imd->func_button = NULL;
+	imd->func_scroll = NULL;
 
 	imd->eventbox = gtk_event_box_new();
 
