@@ -554,73 +554,88 @@ static gint view_window_key_press_cb(GtkWidget *widget, GdkEventKey *event, gpoi
  *-----------------------------------------------------------------------------
  */ 
 
-static void button1_cb(ImageWindow *imd, guint32 time, gdouble x, gdouble y, guint state, gpointer data)
-{
-	ViewWindow *vw = data;
-
-	view_step_next(vw);
-}
-
-static void button2_cb(ImageWindow *imd, guint32 time, gdouble x, gdouble y, guint state, gpointer data)
-{
-	ViewWindow *vw = data;
-
-	view_step_prev(vw);
-}
-
-static void button3_cb(ImageWindow *imd, guint32 time, gdouble x, gdouble y, guint state, gpointer data)
+static void button_cb(ImageWindow *imd, gint button, guint32 time,
+		      gdouble x, gdouble y, guint state, gpointer data)
 {
 	ViewWindow *vw = data;
 	GtkWidget *menu;
 
-	menu = view_popup_menu(vw);
-	gtk_menu_popup (GTK_MENU(menu), NULL, NULL, NULL, NULL, 3, time);
+	switch (button)
+		{
+		case 1:
+			view_step_next(vw);
+			break;
+		case 2:
+			view_step_prev(vw);
+			break;
+		case 3:
+			menu = view_popup_menu(vw);
+			gtk_menu_popup (GTK_MENU(menu), NULL, NULL, NULL, NULL, 3, time);
+			break;
+		default:
+			break;
+		}
 }
 
-static void button4_cb(ImageWindow *imd, guint32 time, gdouble x, gdouble y, guint state, gpointer data)
+static void scroll_cb(ImageWindow *imd, GdkScrollDirection direction, guint32 time,
+		      gdouble x, gdouble y, guint state, gpointer data)
 {
 	ViewWindow *vw = data;
 
 	if (state & GDK_CONTROL_MASK)
 		{
-		image_zoom_adjust_at_point(imd, get_zoom_increment(), x, y);
+		switch (direction)
+			{
+			case GDK_SCROLL_UP:
+				image_zoom_adjust_at_point(imd, get_zoom_increment(), x, y);
+				break;
+			case GDK_SCROLL_DOWN:
+				image_zoom_adjust_at_point(imd, -get_zoom_increment(), x, y);
+				break;
+			default:
+				break;
+			}
 		}
 	else if ( (state & GDK_SHIFT_MASK) != (mousewheel_scrolls))
 		{
-		image_scroll(imd, 0, -MOUSEWHEEL_SCROLL_SIZE);
+		switch (direction)
+			{
+			case GDK_SCROLL_UP:
+				image_scroll(imd, 0, -MOUSEWHEEL_SCROLL_SIZE);
+				break;
+			case GDK_SCROLL_DOWN:
+				image_scroll(imd, 0, MOUSEWHEEL_SCROLL_SIZE);
+				break;
+			case GDK_SCROLL_LEFT:
+				image_scroll(imd, -MOUSEWHEEL_SCROLL_SIZE, 0);
+				break;
+			case GDK_SCROLL_RIGHT:
+				image_scroll(imd, MOUSEWHEEL_SCROLL_SIZE, 0);
+				break;
+			default:
+				break;
+			}
 		}
 	else
 		{
-		view_step_prev(vw);
-		}
-}
-
-static void button5_cb(ImageWindow *imd, guint32 time, gdouble x, gdouble y, guint state, gpointer data)
-{
-	ViewWindow *vw = data;
-
-	if (state & GDK_CONTROL_MASK)
-		{
-		image_zoom_adjust_at_point(imd, -get_zoom_increment(), x, y);
-		}
-	else if ( (state & GDK_SHIFT_MASK) != (mousewheel_scrolls))
-		{
-		image_scroll(imd, 0, MOUSEWHEEL_SCROLL_SIZE);
-		}
-	else
-		{
-		view_step_next(vw);
+		switch (direction)
+			{
+			case GDK_SCROLL_UP:
+				view_step_prev(vw);
+				break;
+			case GDK_SCROLL_DOWN:
+				view_step_next(vw);
+				break;
+			default:
+				break;
+			}
 		}
 }
 
 static void view_image_set_buttons(ViewWindow *vw, ImageWindow *imd)
 {
-	image_set_button_func(imd, 1, button1_cb, vw);
-	image_set_button_func(imd, 2, button2_cb, vw);
-	image_set_button_func(imd, 3, button3_cb, vw);
-	/* for wheel mice */
-	image_set_button_func(imd, 4, button4_cb, vw);
-	image_set_button_func(imd, 5, button5_cb, vw);
+	image_set_button_func(imd, button_cb, vw);
+	image_set_scroll_func(imd, scroll_cb, vw);
 }
 
 static void view_fullscreen_stop_func(FullScreenData *fs, gpointer data)
