@@ -569,21 +569,35 @@ static void info_window_next_cb(GtkWidget *widget, gpointer data)
 	gtk_widget_set_sensitive(id->button_back, TRUE);
 }
 
-static void info_window_image_button_back(ImageWindow *imd, guint32 time,
-					  gdouble x, gdouble y, guint state, gpointer data)
+static void info_window_image_button_cb(ImageWindow *imd, gint button, guint32 time,
+					gdouble x, gdouble y, guint state, gpointer data)
 {
-	info_window_back_cb(NULL, data);
+	if (button == 1)
+		{
+		info_window_next_cb(NULL, data);
+		}
+	else if (button == 2 || button == 3)
+		{
+		info_window_back_cb(NULL, data);
+		}
 }
 
-static void info_window_image_button_next(ImageWindow *imd, guint32 time,
-					  gdouble x, gdouble y, guint state, gpointer data)
+static void info_window_image_scroll_cb(ImageWindow *imd, GdkScrollDirection direction, guint32 time,
+					gdouble x, gdouble y, guint state, gpointer data)
 {
-	info_window_next_cb(NULL, data);
+	if (direction == GDK_SCROLL_UP)
+		{
+		info_window_back_cb(NULL, data);
+		}
+	else if (direction == GDK_SCROLL_DOWN)
+		{
+		info_window_next_cb(NULL, data);
+		}
 }
 
 static void info_window_close(InfoData *id)
 {
-	gdk_window_get_size(id->window->window, &info_window_last_width, &info_window_last_height);
+	gdk_drawable_get_size(id->window->window, &info_window_last_width, &info_window_last_height);
 	info_window_last_width = MAX(info_window_last_width, DEF_PROPERTY_WIDTH);
 	info_window_last_height = MAX(info_window_last_height, DEF_PROPERTY_HEIGHT);
 
@@ -669,10 +683,8 @@ void info_window_new(const gchar *path, GList *list)
 	id->image = image_new(FALSE);
 	image_set_update_func(id->image, info_window_image_update_cb, id);
 
-	image_set_button_func(id->image, 1, info_window_image_button_next, id);
-	image_set_button_func(id->image, 5, info_window_image_button_next, id);
-	image_set_button_func(id->image, 2, info_window_image_button_back, id);
-	image_set_button_func(id->image, 4, info_window_image_button_back, id);
+	image_set_button_func(id->image, info_window_image_button_cb, id);
+	image_set_scroll_func(id->image, info_window_image_scroll_cb, id);
 
 	gtk_widget_set_size_request(id->image->widget, IMAGE_SIZE_W, IMAGE_SIZE_H);
 	gtk_paned_pack1(GTK_PANED(paned), id->image->widget, FALSE, TRUE);
