@@ -863,7 +863,7 @@ gint layout_geometry_get(LayoutWindow *lw, gint *x, gint *y, gint *w, gint *h)
 	if (!layout_valid(&lw)) return FALSE;
 
 	gdk_window_get_root_origin(lw->window->window, x, y);
-	gdk_window_get_size(lw->window->window, w, h);
+	gdk_drawable_get_size(lw->window->window, w, h);
 
 	return TRUE;
 }
@@ -989,7 +989,7 @@ gint layout_geometry_get_tools(LayoutWindow *lw, gint *x, gint *y, gint *w, gint
 		}
 
 	gdk_window_get_root_origin(lw->tools->window, x, y);
-	gdk_window_get_size(lw->tools->window, w, h);
+	gdk_drawable_get_size(lw->tools->window, w, h);
 
 	if (GTK_IS_VPANED(lw->tools_pane))
 		{
