@@ -1209,79 +1209,94 @@ void layout_image_last(LayoutWindow *lw)
  *----------------------------------------------------------------------------
  */
 
-static void layout_image_button1_cb(ImageWindow *imd, guint32 time, gdouble x, gdouble y, guint state, gpointer data)
-{
-	LayoutWindow *lw = data;
-
-	layout_image_next(lw);
-}
-
-static void layout_image_button2_cb(ImageWindow *imd, guint32 time, gdouble x, gdouble y, guint state, gpointer data)
-{
-	LayoutWindow *lw = data;
-
-	layout_image_prev(lw);
-}
-
-static void layout_image_button3_cb(ImageWindow *imd, guint32 time, gdouble x, gdouble y, guint state, gpointer data)
+static void layout_image_button_cb(ImageWindow *imd, gint button, guint32 time,
+				   gdouble x, gdouble y, guint state, gpointer data)
 {
 	LayoutWindow *lw = data;
 	GtkWidget *menu;
 
-	menu = layout_image_pop_menu(lw);
-	gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, 3, time);
-	if (lw->full_screen)
+	switch (button)
 		{
-		/* full screen, re-grab keyboard after popup-menu close */
-		g_signal_connect(G_OBJECT(menu), "selection_done",
-				 G_CALLBACK(layout_image_full_screen_regrab_cb), lw);
+		case 1:
+			layout_image_next(lw);
+			break;
+		case 2:
+			layout_image_prev(lw);
+			break;
+		case 3:
+			menu = layout_image_pop_menu(lw);
+			gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, 3, time);
+			if (lw->full_screen)
+				{
+				/* full screen, re-grab keyboard after popup-menu close */
+				g_signal_connect(G_OBJECT(menu), "selection_done",
+						 G_CALLBACK(layout_image_full_screen_regrab_cb), lw);
+				}
+			break;
+		default:
+			break;
 		}
 }
 
-static void layout_image_button4_cb(ImageWindow *imd, guint32 time, gdouble x, gdouble y, guint state, gpointer data)
+static void layout_image_scroll_cb(ImageWindow *imd, GdkScrollDirection direction, guint32 time,
+				   gdouble x, gdouble y, guint state, gpointer data)
 {
 	LayoutWindow *lw = data;
 
 	if (state & GDK_CONTROL_MASK)
 		{
-		image_zoom_adjust_at_point(imd, get_zoom_increment(), x, y);
+		switch (direction)
+			{
+			case GDK_SCROLL_UP:
+				image_zoom_adjust_at_point(imd, get_zoom_increment(), x, y);
+				break;
+			case GDK_SCROLL_DOWN:
+				image_zoom_adjust_at_point(imd, -get_zoom_increment(), x, y);
+				break;
+			default:
+				break;
+			}
 		}
 	else if ( (state & GDK_SHIFT_MASK) != (mousewheel_scrolls))
 		{
-		image_scroll(imd, 0, -MOUSEWHEEL_SCROLL_SIZE);
+		switch (direction)
+			{
+			case GDK_SCROLL_UP:
+				image_scroll(imd, 0, -MOUSEWHEEL_SCROLL_SIZE);
+				break;
+			case GDK_SCROLL_DOWN:
+				image_scroll(imd, 0, MOUSEWHEEL_SCROLL_SIZE);
+				break;
+			case GDK_SCROLL_LEFT:
+				image_scroll(imd, -MOUSEWHEEL_SCROLL_SIZE, 0);
+				break;
+			case GDK_SCROLL_RIGHT:
+				image_scroll(imd, MOUSEWHEEL_SCROLL_SIZE, 0);
+				break;
+			default:
+				break;
+			}
 		}
 	else
 		{
-		layout_image_prev(lw);
-		}
-}
-
-static void layout_image_button5_cb(ImageWindow *imd, guint32 time, gdouble x, gdouble y, guint state, gpointer data)
-{
-	LayoutWindow *lw = data;
-
-	if (state & GDK_CONTROL_MASK)
-		{
-		image_zoom_adjust_at_point(imd, -get_zoom_increment(), x, y);
-		}
-	else if ( (state & GDK_SHIFT_MASK) != (mousewheel_scrolls))
-		{
-		image_scroll(imd, 0, MOUSEWHEEL_SCROLL_SIZE);
-		}
-	else
-		{
-		layout_image_next(lw);
+		switch (direction)
+			{
+			case GDK_SCROLL_UP:
+				layout_image_prev(lw);
+				break;
+			case GDK_SCROLL_DOWN:
+				layout_image_next(lw);
+				break;
+			default:
+				break;
+			}
 		}
 }
 
 static void layout_image_set_buttons(LayoutWindow *lw)
 {
-	image_set_button_func(lw->image, 1, layout_image_button1_cb, lw);
-	image_set_button_func(lw->image, 2, layout_image_button2_cb, lw);
-	image_set_button_func(lw->image, 3, layout_image_button3_cb, lw);
-	/* for wheel mice */
-	image_set_button_func(lw->image, 4, layout_image_button4_cb, lw);
-	image_set_button_func(lw->image, 5, layout_image_button5_cb, lw);
+	image_set_button_func(lw->image, layout_image_button_cb, lw);
+	image_set_scroll_func(lw->image, layout_image_scroll_cb, lw);
 }
 
 /*
