@@ -1,6 +1,6 @@
 /*
  * GQview
- * (C) 2002 John Ellis
+ * (C) 2004 John Ellis
  *
  * Author: John Ellis
  *
@@ -13,14 +13,18 @@
 #include "gqview.h"
 #include "layout_util.h"
 
+#include "bar_exif.h"
+#include "bar_sort.h"
 #include "cache_maint.h"
 #include "collect.h"
 #include "collect-dlg.h"
 #include "dupe.h"
+#include "editors.h"
 #include "info.h"
 #include "layout_image.h"
 #include "preferences.h"
 #include "utilops.h"
+#include "ui_bookmark.h"
 #include "ui_fileops.h"
 #include "ui_menu.h"
 #include "ui_tabcomp.h"
@@ -28,13 +32,6 @@
 #include <gdk/gdkkeysyms.h> /* for keyboard values */
 
 #include "icons/icon_thumb.xpm"
-#include "icons/icon_home.xpm"
-#include "icons/icon_reload.xpm"
-#include "icons/icon_zoom_in.xpm"
-#include "icons/icon_zoom_out.xpm"
-#include "icons/icon_zoom_fit.xpm"
-#include "icons/icon_zoom_norm.xpm"
-#include "icons/icon_config.xpm"
 #include "icons/icon_float.xpm"
 
 #include "icons/folder_close.xpm"
@@ -51,6 +48,25 @@
  *-----------------------------------------------------------------------------
  */
 
+static guint tree_key_overrides[] = {
+	GDK_Page_Up,	GDK_KP_Page_Up,
+	GDK_Page_Down,	GDK_KP_Page_Down,
+	GDK_Home,	GDK_KP_Home,
+	GDK_End,	GDK_KP_End
+};
+
+static gint layout_key_match(guint keyval)
+{
+	gint i;
+
+	for (i = 0; i < sizeof(tree_key_overrides) / sizeof(guint); i++)
+		{
+		if (keyval == tree_key_overrides[i]) return TRUE;
+		}
+
+	return FALSE;
+}
+
 static gint layout_key_press_cb(GtkWidget *widget, GdkEventKey *event, gpointer data)
 {
 	LayoutWindow *lw = data;
@@ -64,7 +80,20 @@ static gint layout_key_press_cb(GtkWidget *widget, GdkEventKey *event, gpointer
 			{
 			gtk_entry_set_text(GTK_ENTRY(lw->path_entry), lw->path);
 			}
-		return stop_signal;
+
+		/* the gtkaccelgroup of the window is stealing presses before they get to the entry (and more),
+		 * so when the some widgets have focus, give them priority (HACK)
+		 */
+		if (gtk_widget_event(lw->path_entry, (GdkEvent *)event))
+			{
+			return TRUE;
+			}
+		}
+	if (lw->vdt && GTK_WIDGET_HAS_FOCUS(lw->vdt->treeview) &&
+	    !layout_key_match(event->keyval) &&
+	    gtk_widget_event(lw->vdt->treeview, (GdkEvent *)event))
+		{
+		return TRUE;
 		}
 
 	if (lw->image && GTK_WIDGET_HAS_FOCUS(lw->image->widget))
@@ -93,13 +122,18 @@ static gint layout_key_press_cb(GtkWidget *widget, GdkEventKey *event, gpointer
 				stop_signal = TRUE;
 				break;
 			case GDK_space:
+			case 'N': case 'n':
 				layout_image_next(lw);
 				stop_signal = TRUE;
 				break;
+			case GDK_Menu:
+				layout_image_menu_popup(lw);
+				stop_signal = TRUE;
+				break;
 			}
 		}
 
-	if ( !(event->state & GDK_CONTROL_MASK) )
+	if (!stop_signal && !(event->state & GDK_CONTROL_MASK) )
 	    switch (event->keyval)
 		{
 		case '+': case GDK_KP_Add:
@@ -114,7 +148,8 @@ static gint layout_key_press_cb(GtkWidget *widget, GdkEventKey *event, gpointer
 			layout_image_zoom_set(lw, 0.0);
 			stop_signal = TRUE;
 			break;
-		case GDK_KP_Divide: case '1':
+		case GDK_KP_Divide:
+		case '1':
 			layout_image_zoom_set(lw, 1.0);
 			stop_signal = TRUE;
 			break;
@@ -175,12 +210,8 @@ static gint layout_key_press_cb(GtkWidget *widget, GdkEventKey *event, gpointer
 		case 'P': case 'p':
 			layout_image_slideshow_pause_toggle(lw);
 			break;
-		case 'Q': case 'q':
-			if (event->state == 0 || (event->state & GDK_MODIFIER_MASK) == GDK_LOCK_MASK)
-				{
-				exit_gqview();
-				return FALSE;
-				}
+		case 'V': case 'v':
+			layout_image_full_screen_toggle(lw);
 			break;
 		}
 
@@ -196,15 +227,15 @@ static gint layout_key_press_cb(GtkWidget *widget, GdkEventKey *event, gpointer
 		layout_image_scroll(lw, x, y);
 		}
 
-	if (stop_signal) gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), "key_press_event");
+	if (stop_signal) g_signal_stop_emission_by_name(G_OBJECT(widget), "key_press_event");
 
 	return stop_signal;
 }
 
 void layout_keyboard_init(LayoutWindow *lw)
 {
-	gtk_signal_connect(GTK_OBJECT(lw->window), "key_press_event",
-			   GTK_SIGNAL_FUNC(layout_key_press_cb), lw);
+	g_signal_connect(G_OBJECT(lw->window), "key_press_event",
+			 G_CALLBACK(layout_key_press_cb), lw);
 }
 
 /*
@@ -329,6 +360,7 @@ static void layout_menu_info_cb(gpointer data, guint action, GtkWidget *widget)
 	list = layout_selection_list(lw);
 	if (!list) path = layout_image_get_path(lw);
 
+	layout_image_full_screen_stop(lw);
 	info_window_new(path, list);
 }
 
@@ -391,13 +423,6 @@ static void layout_menu_zoom_fit_cb(gpointer data, guint action, GtkWidget *widg
 	layout_image_zoom_set(lw, 0.0);
 }
 
-static void layout_menu_full_screen_cb(gpointer data, guint action, GtkWidget *widget)
-{
-	LayoutWindow *lw = data;
-
-	layout_image_full_screen_start(lw);
-}
-
 static void layout_menu_thumb_cb(gpointer data, guint action, GtkWidget *widget)
 {
 	LayoutWindow *lw = data;
@@ -405,6 +430,24 @@ static void layout_menu_thumb_cb(gpointer data, guint action, GtkWidget *widget)
 	layout_thumb_set(lw, GTK_CHECK_MENU_ITEM(widget)->active);
 }
 
+static void layout_menu_list_cb(gpointer data, guint action, GtkWidget *widget)
+{
+	LayoutWindow *lw = data;
+
+	if (!GTK_CHECK_MENU_ITEM(widget)->active) return;
+
+	layout_views_set(lw, lw->tree_view, FALSE);
+}
+
+static void layout_menu_icon_cb(gpointer data, guint action, GtkWidget *widget)
+{
+	LayoutWindow *lw = data;
+
+	if (!GTK_CHECK_MENU_ITEM(widget)->active) return;
+
+	layout_views_set(lw, lw->tree_view, TRUE);
+}
+
 static void layout_menu_tree_cb(gpointer data, guint action, GtkWidget *widget)
 {
 	LayoutWindow *lw = data;
@@ -412,11 +455,11 @@ static void layout_menu_tree_cb(gpointer data, guint action, GtkWidget *widget)
 	layout_views_set(lw, GTK_CHECK_MENU_ITEM(widget)->active, lw->icon_view);
 }
 
-static void layout_menu_icon_cb(gpointer data, guint action, GtkWidget *widget)
+static void layout_menu_full_screen_cb(gpointer data, guint action, GtkWidget *widget)
 {
 	LayoutWindow *lw = data;
 
-	layout_views_set(lw, lw->tree_view, GTK_CHECK_MENU_ITEM(widget)->active);
+	layout_image_full_screen_start(lw);
 }
 
 static void layout_menu_refresh_cb(gpointer data, guint action, GtkWidget *widget)
@@ -453,6 +496,26 @@ static void layout_menu_toolbar_cb(gpointer data, guint action, GtkWidget *widge
 		}
 }
 
+static void layout_menu_bar_exif_cb(gpointer data, guint action, GtkWidget *widget)
+{
+	LayoutWindow *lw = data;
+
+	if (lw->bar_exif_enabled != GTK_CHECK_MENU_ITEM(widget)->active)
+		{
+		layout_bar_exif_toggle(lw);
+		}
+}
+
+static void layout_menu_bar_sort_cb(gpointer data, guint action, GtkWidget *widget)
+{
+	LayoutWindow *lw = data;
+
+	if (lw->bar_sort_enabled != GTK_CHECK_MENU_ITEM(widget)->active)
+		{
+		layout_bar_sort_toggle(lw);
+		}
+}
+
 static void layout_menu_slideshow_cb(gpointer data, guint action, GtkWidget *widget)
 {
 	LayoutWindow *lw = data;
@@ -484,11 +547,14 @@ static void layout_menu_about_cb(gpointer data, guint action, GtkWidget *widget)
 static void layout_menu_edit_cb(gpointer data, guint action, GtkWidget *widget)
 {
 	LayoutWindow *lw = data;
+	GList *list;
 	gint n;
 
-	n = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(widget), "edit_index"));
+	n = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(widget), "edit_index"));
 
-	start_editor_from_file(n, layout_image_get_path(lw));
+	list = layout_selection_list(lw);
+	start_editor_from_path_list(n, list);
+	path_list_free(list);
 }
 
 static void layout_menu_edit_update(LayoutWindow *lw)
@@ -497,12 +563,12 @@ static void layout_menu_edit_update(LayoutWindow *lw)
 
 	/* main edit menu */
 
-	for(i = 0; i < 8; i++)
+	for(i = 0; i < 10; i++)
 		{
 		GtkWidget *item;
 
 		item = gtk_item_factory_get_item_by_action(lw->menu_fact, MENU_EDIT_ACTION_OFFSET + i);
-		gtk_object_set_data(GTK_OBJECT(item), "edit_index", GINT_TO_POINTER(i));
+		g_object_set_data(G_OBJECT(item), "edit_index", GINT_TO_POINTER(i));
 
 		if (editor_command[i] && strlen(editor_command[i]) > 0)
 			{
@@ -551,7 +617,7 @@ static void layout_menu_recent_cb(GtkWidget *widget, gpointer data)
 	gint n;
 	gchar *path;
 
-	n = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(widget), "recent_index"));
+	n = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(widget), "recent_index"));
 
 	path = g_list_nth_data(history_list_get_by_key("recent"), n);
 
@@ -578,9 +644,9 @@ static void layout_menu_recent_update(LayoutWindow *lw)
 
 	while (list)
 		{
-		item = menu_item_add(menu, filename_from_path((gchar *)list->data),
-				     GTK_SIGNAL_FUNC(layout_menu_recent_cb), lw);
-		gtk_object_set_data(GTK_OBJECT(item), "recent_index", GINT_TO_POINTER(n));
+		item = menu_item_add_simple(menu, filename_from_path((gchar *)list->data),
+					    G_CALLBACK(layout_menu_recent_cb), lw);
+		g_object_set_data(G_OBJECT(item), "recent_index", GINT_TO_POINTER(n));
 		list = list->next;
 		n++;
 		}
@@ -628,22 +694,22 @@ static GtkItemFactoryEntry gqview_menu_items[] =
 {
 	{ N_("/_File"),				NULL,		NULL,				0, "<Branch>" },
 	{ N_("/File/tear1"),			NULL,		NULL,				0, "<Tearoff>" },
-	{ N_("/File/New _window"),		NULL, 		layout_menu_new_window_cb,	0, NULL },
-	{ N_("/File/_New collection"),		"C",		layout_menu_new_cb,		0, NULL },
-	{ N_("/File/_Open collection..."),	"O",		layout_menu_open_cb,		0, NULL },
+	{ N_("/File/New _window"),		NULL, 		layout_menu_new_window_cb,	0, "<StockItem>", GTK_STOCK_NEW },
+	{ N_("/File/_New collection"),		"C",		layout_menu_new_cb,		0, "<StockItem>", GTK_STOCK_INDEX },
+	{ N_("/File/_Open collection..."),	"O",		layout_menu_open_cb,		0, "<StockItem>", GTK_STOCK_OPEN },
 	{ N_("/File/Open _recent"),		NULL,		NULL,				0, NULL },
 	{ N_("/File/sep1"),			NULL,		NULL,				0, "<Separator>" },
-	{ N_("/File/_Find duplicates..."),	"D",		layout_menu_dupes_cb,		0, NULL },
+	{ N_("/File/_Find duplicates..."),	"D",		layout_menu_dupes_cb,		0, "<StockItem>", GTK_STOCK_FIND },
 	{ N_("/File/sep2"),			NULL,		NULL,				0, "<Separator>" },
-	{ N_("/File/Create _Dir..."),		"<control>N",	layout_menu_dir_cb,		0, NULL },
+	{ N_("/File/N_ew folder..."),		"<control>F",	layout_menu_dir_cb,		0, NULL },
 	{ N_("/File/sep3"),			NULL,		NULL,				0, "<Separator>" },
-	{ N_("/File/_Copy..."),			"<control>C",	layout_menu_copy_cb,		0, NULL },
+	{ N_("/File/_Copy..."),			"<control>C",	layout_menu_copy_cb,		0, "<StockItem>", GTK_STOCK_COPY },
 	{ N_("/File/_Move..."),			"<control>M",	layout_menu_move_cb,		0, NULL },
 	{ N_("/File/_Rename..."),		"<control>R",	layout_menu_rename_cb,		0, NULL },
-	{ N_("/File/_Delete..."),		"<control>D",	layout_menu_delete_cb,		0, NULL },
+	{ N_("/File/_Delete..."),		"<control>D",	layout_menu_delete_cb,		0, "<StockItem>", GTK_STOCK_DELETE },
 	{ N_("/File/sep4"),			NULL,		NULL,				0, "<Separator>" },
-	{ N_("/File/C_lose window"),		"<control>W",	layout_menu_close_cb,		0, NULL },
-	{ N_("/File/E_xit"),			"<control>Q",	layout_menu_exit_cb,		0, NULL },
+	{ N_("/File/C_lose window"),		"<control>W",	layout_menu_close_cb,		0, "<StockItem>", GTK_STOCK_CLOSE },
+	{ N_("/File/E_xit"),			"<control>Q",	layout_menu_exit_cb,		0, "<StockItem>", GTK_STOCK_QUIT },
 
 	{ N_("/_Edit"),				NULL,		NULL,				0, "<Branch>" },
 	{ N_("/Edit/tear1"),			NULL,		NULL,				0, "<Tearoff>" },
@@ -655,9 +721,11 @@ static GtkItemFactoryEntry gqview_menu_items[] =
 	{ N_("/Edit/editor6"),			"<control>6",	layout_menu_edit_cb,		MENU_EDIT_ACTION_OFFSET + 5, NULL },
 	{ N_("/Edit/editor7"),			"<control>7",	layout_menu_edit_cb,		MENU_EDIT_ACTION_OFFSET + 6, NULL },
 	{ N_("/Edit/editor8"),			"<control>8",	layout_menu_edit_cb,		MENU_EDIT_ACTION_OFFSET + 7, NULL },
+	{ N_("/Edit/editor9"),			"<control>9",	layout_menu_edit_cb,		MENU_EDIT_ACTION_OFFSET + 8, NULL },
+	{ N_("/Edit/editor0"),			"<control>0",	layout_menu_edit_cb,		MENU_EDIT_ACTION_OFFSET + 9, NULL},
 	{ N_("/Edit/sep1"),			NULL,		NULL,				0, "<Separator>" },
 	{ N_("/Edit/_Adjust"),			NULL,		NULL,				0, "<Branch>" },
-	{ N_("/Edit/_Properties"),		"<control>P",	layout_menu_info_cb,		0, NULL },
+	{ N_("/Edit/_Properties"),		"<control>P",	layout_menu_info_cb,		0, "<StockItem>", GTK_STOCK_PROPERTIES },
 	{ N_("/Edit/Adjust/tear1"),		NULL,		NULL,				0, "<Tearoff>" },
 	{ N_("/Edit/Adjust/_Rotate clockwise"),		"bracketright",	layout_menu_alter_90_cb,	0, NULL },
 	{ N_("/Edit/Adjust/Rotate _counterclockwise"),	"bracketleft",	layout_menu_alter_90cc_cb,	0, NULL },
@@ -668,37 +736,41 @@ static GtkItemFactoryEntry gqview_menu_items[] =
 	{ N_("/Edit/Select _all"),		"<control>A",	layout_menu_select_all_cb,	0, NULL },
 	{ N_("/Edit/Select _none"),	"<control><shift>A",	layout_menu_unselect_all_cb,	0, NULL },
 	{ N_("/Edit/sep3"),			NULL,		NULL,				0, "<Separator>" },
-	{ N_("/Edit/_Options..."),		"<control>O",	layout_menu_config_cb,		0, NULL },
+	{ N_("/Edit/_Options..."),		"<control>O",	layout_menu_config_cb,		0, "<StockItem>", GTK_STOCK_PREFERENCES },
 	{ N_("/Edit/_Remove old thumbnails"),	NULL,		layout_menu_remove_thumb_cb,	0, NULL },
 	{ N_("/Edit/sep4"),			NULL,		NULL,				0, "<Separator>" },
-	{ N_("/Edit/Set as _wallpaper"),	"<control>W",	layout_menu_wallpaper_cb,	0, NULL },
+	{ N_("/Edit/Set as _wallpaper"),	NULL,		layout_menu_wallpaper_cb,	0, NULL },
 
 	{ N_("/_View"),				NULL,		NULL,				0, "<Branch>" },
 	{ N_("/View/tear1"),			NULL,		NULL,				0, "<Tearoff>" },
-	{ N_("/View/Zoom _in"),			"equal",	layout_menu_zoom_in_cb,		0, NULL },
-	{ N_("/View/Zoom _out"),		"minus",	layout_menu_zoom_out_cb,	0, NULL },
-	{ N_("/View/Zoom _1:1"),		"Z",		layout_menu_zoom_1_1_cb,	0, NULL },
-	{ N_("/View/_Zoom to fit"),		"X",		layout_menu_zoom_fit_cb,	0, NULL },
+	{ N_("/View/Zoom _in"),			"equal",	layout_menu_zoom_in_cb,		0, "<StockItem>", GTK_STOCK_ZOOM_IN },
+	{ N_("/View/Zoom _out"),		"minus",	layout_menu_zoom_out_cb,	0, "<StockItem>", GTK_STOCK_ZOOM_OUT },
+	{ N_("/View/Zoom _1:1"),		"Z",		layout_menu_zoom_1_1_cb,	0, "<StockItem>", GTK_STOCK_ZOOM_100 },
+	{ N_("/View/_Zoom to fit"),		"X",		layout_menu_zoom_fit_cb,	0, "<StockItem>", GTK_STOCK_ZOOM_FIT },
 	{ N_("/View/sep1"),			NULL,		NULL,				0, "<Separator>" },
 
-	{ N_("/View/F_ull screen"),		"V",		layout_menu_full_screen_cb,	0, NULL },
 	{ N_("/View/_Thumbnails"),		"T",		layout_menu_thumb_cb,		0, "<CheckItem>" },
+	{ N_("/View/_List"),			"<control>L",	layout_menu_list_cb,		0, "<RadioItem>" },
+	{ N_("/View/I_cons"),			"<control>I",	layout_menu_icon_cb,		0, "/View/List" },
 
 	{ N_("/View/sep2"),			NULL,		NULL,				0, "<Separator>" },
 	{ N_("/View/Tr_ee"),			"<control>T",	layout_menu_tree_cb,		0, "<CheckItem>" },
-	{ N_("/View/I_cons"),			"<control>I",	layout_menu_icon_cb,		0, "<CheckItem>" },
+	{ N_("/View/F_ull screen"),		"F",		layout_menu_full_screen_cb,	0, NULL },
 
 	{ N_("/View/sep3"),			NULL,		NULL,				0, "<Separator>" },
-	{ N_("/View/_Float file list"),		"F",		layout_menu_float_cb,		0, "<CheckItem>" },
+	{ N_("/View/_Float file list"),		"L",		layout_menu_float_cb,		0, "<CheckItem>" },
 	{ N_("/View/_Hide file list"),		"H",		layout_menu_hide_cb,		0, NULL },
 	{ N_("/View/Hide tool_bar"),		NULL,		layout_menu_toolbar_cb,		0, "<CheckItem>" },
 	{ N_("/View/sep4"),			NULL,		NULL,				0, "<Separator>" },
+	{ N_("/View/E_xif data"),		"<control>E",	layout_menu_bar_exif_cb,	0, "<CheckItem>" },
+	{ N_("/View/Sort _manager"),		"<control>S",	layout_menu_bar_sort_cb,	0, "<CheckItem>" },
+	{ N_("/View/sep5"),			NULL,		NULL,				0, "<Separator>" },
 	{ N_("/View/Toggle _slideshow"),	"S",		layout_menu_slideshow_cb,	0, NULL },
-	{ N_("/View/_Refresh Lists"),		"R",		layout_menu_refresh_cb,		0, NULL },
+	{ N_("/View/_Refresh Lists"),		"R",		layout_menu_refresh_cb,		0, "<StockItem>", GTK_STOCK_REFRESH },
 
 	{ N_("/_Help"),				NULL,		NULL,				0, "<Branch>" },
 	{ N_("/Help/tear1"),			NULL,		NULL,				0, "<Tearoff>" },
-	{ N_("/Help/_Keyboard shortcuts"),	"F1",		layout_menu_help_cb,		0, NULL},
+	{ N_("/Help/_Keyboard shortcuts"),	"F1",		layout_menu_help_cb,		0, "<StockItem>", GTK_STOCK_HELP },
 	{ N_("/Help/_Release notes"),		NULL,		layout_menu_notes_cb,		0, NULL},
 	{ N_("/Help/sep1"),			NULL,		NULL,				0, "<Separator>" },
 	{ N_("/Help/_About"),			NULL,		layout_menu_about_cb,		0, NULL },
@@ -789,15 +861,12 @@ static void layout_button_float_cb(GtkWidget *widget, gpointer data)
 	layout_tools_float_toggle(lw);
 }
 
-GtkWidget *layout_button(GtkWidget *box, gchar **pixmap_data, gint toggle,
+GtkWidget *layout_button(GtkWidget *box, gchar **pixmap_data, const gchar *stock_id, gint toggle,
 			 GtkTooltips *tooltips, const gchar *tip_text,
-			 GtkSignalFunc func, gpointer data)
+			 GCallback func, gpointer data)
 {
 	GtkWidget *button;
 	GtkWidget *icon;
-	GdkPixbuf *pixbuf;
-	GdkPixmap *pixmap;
-	GdkBitmap *mask;
 
 	if (toggle)
 		{
@@ -808,21 +877,28 @@ GtkWidget *layout_button(GtkWidget *box, gchar **pixmap_data, gint toggle,
 		button = gtk_button_new();
 		}
 
-	gtk_signal_connect(GTK_OBJECT(button), "clicked", func, data);
+	g_signal_connect(G_OBJECT(button), "clicked", func, data);
 	gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 0);
 	gtk_widget_show(button);
 	gtk_tooltips_set_tip(tooltips, button, tip_text, NULL);
 
-	pixbuf = gdk_pixbuf_new_from_xpm_data((const char **)pixmap_data);
-	gdk_pixbuf_render_pixmap_and_mask(pixbuf, &pixmap, &mask, 128);
-	gdk_pixbuf_unref(pixbuf);
+	if (stock_id)
+		{
+		icon = gtk_image_new_from_stock(stock_id, GTK_ICON_SIZE_BUTTON);
+		}
+	else
+		{
+		GdkPixbuf *pixbuf;
+
+		pixbuf = gdk_pixbuf_new_from_xpm_data((const char **)pixmap_data);
+		icon = gtk_image_new_from_pixbuf(pixbuf);
+		gdk_pixbuf_unref(pixbuf);
+		}
+
+	gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
 
-	icon = gtk_pixmap_new(pixmap, mask);
-	gtk_widget_show(icon);
 	gtk_container_add(GTK_CONTAINER(button), icon);
-
-	gdk_pixmap_unref(pixmap);
-	if (mask) gdk_bitmap_unref(mask);
+	gtk_widget_show(icon);
 
 	return button;
 }
@@ -836,24 +912,24 @@ GtkWidget *layout_button_bar(LayoutWindow *lw)
 
 	box = gtk_hbox_new(FALSE, 0);
 
-	lw->thumb_button = layout_button(box, (gchar **)icon_thumb_xpm, TRUE,
-		      tooltips, _("Show thumbnails"), GTK_SIGNAL_FUNC(layout_button_thumb_cb), lw);
-	layout_button(box, (gchar **)icon_home_xpm, FALSE,
-		      tooltips, _("Change to home directory"), GTK_SIGNAL_FUNC(layout_button_home_cb), lw);
-	layout_button(box, (gchar **)icon_reload_xpm, FALSE,
-		      tooltips, _("Refresh file list"), GTK_SIGNAL_FUNC(layout_button_refresh_cb), lw);
-	layout_button(box, (gchar **)icon_zoom_in_xpm, FALSE,
-		      tooltips, _("Zoom in"), GTK_SIGNAL_FUNC(layout_button_zoom_in_cb), lw);
-	layout_button(box, (gchar **)icon_zoom_out_xpm, FALSE,
-		      tooltips, _("Zoom out"), GTK_SIGNAL_FUNC(layout_button_zoom_out_cb), lw);
-	layout_button(box, (gchar **)icon_zoom_fit_xpm, FALSE,
-		      tooltips, _("Fit image to window"), GTK_SIGNAL_FUNC(layout_button_zoom_fit_cb), lw);
-	layout_button(box, (gchar **)icon_zoom_norm_xpm, FALSE,
-		      tooltips, _("Set zoom 1:1"), GTK_SIGNAL_FUNC(layout_button_zoom_1_1_cb), lw);
-	layout_button(box, (gchar **)icon_config_xpm, FALSE,
-		      tooltips, _("Configure options"), GTK_SIGNAL_FUNC(layout_button_config_cb), lw);
-	layout_button(box, (gchar **)icon_float_xpm, FALSE,
-		      tooltips, _("Float Controls"), GTK_SIGNAL_FUNC(layout_button_float_cb), lw);
+	lw->thumb_button = layout_button(box, (gchar **)icon_thumb_xpm, NULL, TRUE,
+		      tooltips, _("Show thumbnails"), G_CALLBACK(layout_button_thumb_cb), lw);
+	layout_button(box, NULL, GTK_STOCK_HOME, FALSE,
+		      tooltips, _("Change to home directory"), G_CALLBACK(layout_button_home_cb), lw);
+	layout_button(box, NULL, GTK_STOCK_REFRESH, FALSE,
+		      tooltips, _("Refresh file list"), G_CALLBACK(layout_button_refresh_cb), lw);
+	layout_button(box, NULL, GTK_STOCK_ZOOM_IN, FALSE,
+		      tooltips, _("Zoom in"), G_CALLBACK(layout_button_zoom_in_cb), lw);
+	layout_button(box, NULL, GTK_STOCK_ZOOM_OUT, FALSE,
+		      tooltips, _("Zoom out"), G_CALLBACK(layout_button_zoom_out_cb), lw);
+	layout_button(box, NULL, GTK_STOCK_ZOOM_FIT, FALSE,
+		      tooltips, _("Fit image to window"), G_CALLBACK(layout_button_zoom_fit_cb), lw);
+	layout_button(box, NULL, GTK_STOCK_ZOOM_100, FALSE,
+		      tooltips, _("Set zoom 1:1"), G_CALLBACK(layout_button_zoom_1_1_cb), lw);
+	layout_button(box, NULL, GTK_STOCK_PREFERENCES, FALSE,
+		      tooltips, _("Configure options"), G_CALLBACK(layout_button_config_cb), lw);
+	layout_button(box, (gchar **)icon_float_xpm, NULL, FALSE,
+		      tooltips, _("Float Controls"), G_CALLBACK(layout_button_float_cb), lw);
 
 	return box;
 }
@@ -868,29 +944,50 @@ static void layout_util_sync_views(LayoutWindow *lw)
 {
 	GtkWidget *item;
 
+	if (!lw->menu_fact) return;
+
 	item = gtk_item_factory_get_item(lw->menu_fact, "/View/Tree");
-	gtk_check_menu_item_set_state(GTK_CHECK_MENU_ITEM(item), lw->tree_view);
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item), lw->tree_view);
 
 	item = gtk_item_factory_get_item(lw->menu_fact, "/View/Icons");
-	gtk_check_menu_item_set_state(GTK_CHECK_MENU_ITEM(item), lw->icon_view);
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item), lw->icon_view);
 
 	item = gtk_item_factory_get_item(lw->menu_fact, "/View/Float file list");
-	gtk_check_menu_item_set_state(GTK_CHECK_MENU_ITEM(item), lw->tools_float);
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item), lw->tools_float);
+
+	item = gtk_item_factory_get_item(lw->menu_fact, "/View/Exif data");
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item), lw->bar_exif_enabled);
+
+	item = gtk_item_factory_get_item(lw->menu_fact, "/View/Sort manager");
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item), lw->bar_sort_enabled);
 
 	item = gtk_item_factory_get_item(lw->menu_fact, "/View/Hide toolbar");
-	gtk_check_menu_item_set_state(GTK_CHECK_MENU_ITEM(item), lw->toolbar_hidden);
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item), lw->toolbar_hidden);
 }
 
 void layout_util_sync_thumb(LayoutWindow *lw)
 {
 	GtkWidget *item;
 
+	if (lw->icon_view)
+		{
+		item = gtk_item_factory_get_item(lw->menu_fact, "/View/Icons");
+		}
+	else
+		{
+		item = gtk_item_factory_get_item(lw->menu_fact, "/View/List");
+		}
+
+	if (!GTK_CHECK_MENU_ITEM(item)->active)
+		{
+		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item), TRUE);
+		}
+
 	item = gtk_item_factory_get_item(lw->menu_fact, "/View/Thumbnails");
-	gtk_check_menu_item_set_state(GTK_CHECK_MENU_ITEM(item), lw->thumbs_enabled);
+	GTK_CHECK_MENU_ITEM(item)->active = lw->thumbs_enabled;
 	gtk_widget_set_sensitive(item, !lw->icon_view);
 
-	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(lw->thumb_button), lw->thumbs_enabled);
-
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lw->thumb_button), lw->thumbs_enabled);
 	gtk_widget_set_sensitive(lw->thumb_button, !lw->icon_view);
 }
 
@@ -911,43 +1008,185 @@ void layout_util_sync(LayoutWindow *lw)
 PixmapFolders *folder_icons_new(void)
 {
 	PixmapFolders *pf;
-	GdkPixbuf *pixbuf;
 
 	pf = g_new0(PixmapFolders, 1);
 
-	pixbuf = gdk_pixbuf_new_from_xpm_data((const char **)folder_close_xpm);
-	gdk_pixbuf_render_pixmap_and_mask(pixbuf, &pf->close_pixmap, &pf->close_mask, 128);
-	gdk_pixbuf_unref(pixbuf);
-
-	pixbuf = gdk_pixbuf_new_from_xpm_data((const char **)folder_open_xpm);
-	gdk_pixbuf_render_pixmap_and_mask(pixbuf, &pf->open_pixmap, &pf->open_mask, 128);
-	gdk_pixbuf_unref(pixbuf);
-
-	pixbuf = gdk_pixbuf_new_from_xpm_data((const char **)folder_deny_xpm);
-	gdk_pixbuf_render_pixmap_and_mask(pixbuf, &pf->deny_pixmap, &pf->deny_mask, 128);
-	gdk_pixbuf_unref(pixbuf);
+	pf->close = gdk_pixbuf_new_from_xpm_data((const char **)folder_close_xpm);
+	pf->open = gdk_pixbuf_new_from_xpm_data((const char **)folder_open_xpm);
+	pf->deny = gdk_pixbuf_new_from_xpm_data((const char **)folder_deny_xpm);
 
 	return pf;
 }
 
-static void folder_icon_unref(GdkPixmap *pixmap)
-{
-	if (pixmap) gdk_pixmap_unref(pixmap);
-}
-
 void folder_icons_free(PixmapFolders *pf)
 {
 	if (!pf) return;
 
-	folder_icon_unref(pf->close_pixmap);
-	folder_icon_unref(pf->close_mask);
-
-	folder_icon_unref(pf->open_pixmap);
-	folder_icon_unref(pf->open_mask);
-
-	folder_icon_unref(pf->deny_pixmap);
-	folder_icon_unref(pf->deny_mask);
+	g_object_unref(pf->close);
+	g_object_unref(pf->open);
+	g_object_unref(pf->deny);
 
 	g_free(pf);
 }
 
+/*
+ *-----------------------------------------------------------------------------
+ * sidebars
+ *-----------------------------------------------------------------------------
+ */
+
+#define SIDEBAR_WIDTH 288
+
+static void layout_bar_exif_destroyed(GtkWidget *widget, gpointer data)
+{
+	LayoutWindow *lw = data;
+
+	if (lw->bar_exif)
+		{
+		lw->bar_exif_advanced = bar_exif_is_advanced(lw->bar_exif);
+		}
+
+	lw->bar_exif = NULL;
+	if (lw->utility_box)
+		{
+		/* destroyed from within itself */
+		lw->bar_exif_enabled = FALSE;
+		layout_util_sync_views(lw);
+		}
+}
+
+static void layout_bar_exif_sized(GtkWidget *widget, GtkAllocation *allocation, gpointer data)
+{
+	LayoutWindow *lw = data;
+
+	if (lw->bar_exif)
+		{
+		lw->bar_exif_size = allocation->width;
+		}
+}
+
+void layout_bar_exif_new(LayoutWindow *lw)
+{
+	if (!lw->utility_box) return;
+
+	lw->bar_exif = bar_exif_new(TRUE, layout_image_get_path(lw), lw->bar_exif_advanced);
+	g_signal_connect(G_OBJECT(lw->bar_exif), "destroy",
+			 G_CALLBACK(layout_bar_exif_destroyed), lw);
+	g_signal_connect(G_OBJECT(lw->bar_exif), "size_allocate",
+			 G_CALLBACK(layout_bar_exif_sized), lw);
+        lw->bar_exif_enabled = TRUE;
+
+	if (lw->bar_exif_size < 1) lw->bar_exif_size = SIDEBAR_WIDTH;
+	gtk_widget_set_size_request(lw->bar_exif, lw->bar_exif_size, -1);
+	gtk_box_pack_start(GTK_BOX(lw->utility_box), lw->bar_exif, FALSE, FALSE, 0);
+	gtk_widget_show(lw->bar_exif);
+}
+
+void layout_bar_exif_close(LayoutWindow *lw)
+{
+	if (lw->bar_exif)
+		{
+		bar_exif_close(lw->bar_exif);
+		lw->bar_exif = NULL;
+		}
+        lw->bar_exif_enabled = FALSE;
+}
+
+void layout_bar_exif_toggle(LayoutWindow *lw)
+{
+	if (lw->bar_exif_enabled)
+		{
+		layout_bar_exif_close(lw);
+		}
+	else
+		{
+		layout_bar_exif_new(lw);
+		}
+}
+
+static void layout_bar_exif_new_image(LayoutWindow *lw)
+{
+	if (!lw->bar_exif || !lw->bar_exif_enabled) return;
+
+	bar_exif_set(lw->bar_exif, layout_image_get_path(lw));
+}
+
+static void layout_bar_sort_destroyed(GtkWidget *widget, gpointer data)
+{
+	LayoutWindow *lw = data;
+
+	lw->bar_sort = NULL;
+
+	if (lw->utility_box)
+		{
+		/* destroyed from within itself */
+		lw->bar_sort_enabled = FALSE;
+
+		layout_util_sync_views(lw);
+		}
+}
+
+void layout_bar_sort_new(LayoutWindow *lw)
+{
+	if (!lw->utility_box) return;
+
+	lw->bar_sort = bar_sort_new(lw);
+	g_signal_connect(G_OBJECT(lw->bar_sort), "destroy",
+			 G_CALLBACK(layout_bar_sort_destroyed), lw);
+	lw->bar_sort_enabled = TRUE;
+
+	gtk_box_pack_end(GTK_BOX(lw->utility_box), lw->bar_sort, FALSE, FALSE, 0);
+	gtk_widget_show(lw->bar_sort);
+}
+
+void layout_bar_sort_close(LayoutWindow *lw)
+{
+	if (lw->bar_sort)
+		{
+		bar_sort_close(lw->bar_sort);
+		lw->bar_sort = NULL;
+		}
+	lw->bar_sort_enabled = FALSE;
+}
+
+void layout_bar_sort_toggle(LayoutWindow *lw)
+{
+	if (lw->bar_sort_enabled)
+		{
+		layout_bar_sort_close(lw);
+		}
+	else
+		{
+		layout_bar_sort_new(lw);
+		}
+}
+
+void layout_bars_new_image(LayoutWindow *lw)
+{
+	layout_bar_exif_new_image(lw);
+}
+
+GtkWidget *layout_bars_prepare(LayoutWindow *lw, GtkWidget *image)
+{
+	lw->utility_box = gtk_hbox_new(FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(lw->utility_box), image, TRUE, TRUE, 0);
+	gtk_widget_show(image);
+
+	if (lw->bar_sort_enabled)
+		{
+		layout_bar_sort_new(lw);
+		}
+
+	if (lw->bar_exif_enabled)
+		{
+		layout_bar_exif_new(lw);
+		}
+
+	return lw->utility_box;
+}
+
+void layout_bars_close(LayoutWindow *lw)
+{
+	layout_bar_sort_close(lw);
+	layout_bar_exif_close(lw);
+}
