@@ -57,16 +57,25 @@ static void encoding_dialog(const gchar *path)
 	GenericDialog *gd;
 	GString *string;
 	const gchar *lc;
+	const gchar *bf;
+
+	/* check that gtk is initialized (loop is level > 0) */
+	if (gtk_main_level() == 0) return;
 
 	if (warned_user) return;
 	warned_user = TRUE;
 
 	lc = getenv("LANG");
+	bf = getenv("G_BROKEN_FILENAMES");
 	warned_user = TRUE;
 
 	string = g_string_new("");
-	string = g_string_append(string, "One or more filenames are not encoded with the preferred locale character set.\n");
-	string = g_string_append(string, "Operations on, and display of these files with GQview may not succeed.\n");
+	g_string_append(string, "One or more filenames are not encoded with the preferred locale character set.\n");
+	g_string_append_printf(string, "Operations on, and display of these files with %s may not succeed.\n\n", PACKAGE);
+	g_string_append(string, "If your filenames are not encoded in utf-8, try setting\n");
+	g_string_append(string, "the environment variable G_BROKEN_FILENAMES=1\n");
+	g_string_append_printf(string, "It appears G_BROKEN_FILENAMES is %s%s\n\n",
+				(bf) ? "set to " : "not set.", (bf) ? bf : "");
 	g_string_append_printf(string, "The locale appears to be set to \"%s\"\n(set by the LANG environment variable)\n", (lc) ? lc : "undefined");
 	if (lc && (strstr(lc, "UTF-8") || strstr(lc, "utf-8")))
 		{
@@ -80,7 +89,7 @@ static void encoding_dialog(const gchar *path)
 		}
 
 	gd = generic_dialog_new("Filename encoding locale mismatch", string->str,
-				"gqview", "locale warning", TRUE, NULL, NULL);
+				PACKAGE, "locale warning", TRUE, NULL, NULL);
 	generic_dialog_add_stock(gd, NULL, GTK_STOCK_CLOSE, NULL, TRUE);
 	gtk_widget_show(gd->dialog);
 
@@ -94,10 +103,10 @@ gchar *path_to_utf8(const gchar *path)
 
 	if (!path) return NULL;
 
-	utf8 = g_locale_to_utf8(path, -1, NULL, NULL, &error);
+	utf8 = g_filename_to_utf8(path, -1, NULL, NULL, &error);
 	if (error)
 		{
-		printf("Unable to convert to UTF-8 from locale:\n%s\n%s\n", path, error->message);
+		printf("Unable to convert filename to UTF-8:\n%s\n%s\n", path, error->message);
 		g_error_free(error);
 		encoding_dialog(path);
 		}
@@ -117,10 +126,10 @@ gchar *path_from_utf8(const gchar *utf8)
 
 	if (!utf8) return NULL;
 
-	path = g_locale_from_utf8(utf8, -1, NULL, NULL, &error);
+	path = g_filename_from_utf8(utf8, -1, NULL, NULL, &error);
 	if (error)
 		{
-		printf("Unable to convert to locale from UTF-8:\n%s\n%s\n", utf8, error->message);
+		printf("Unable to convert filename to locale from UTF-8:\n%s\n%s\n", utf8, error->message);
 		g_error_free(error);
 		}
 	if (!path)
