@@ -1294,16 +1294,30 @@ static void file_util_delete_multiple_review_next(GtkWidget *button, gpointer da
 	file_util_delete_multiple_review_skip(gd, TRUE);
 }
 
-static void file_util_delete_multiple_review_btn_back(ImageWindow *imd, guint32 time,
-						      gdouble x, gdouble y, guint state, gpointer data)
+static void file_util_delete_multiple_review_button_cb(ImageWindow *imd, gint button, guint32 time,
+						       gdouble x, gdouble y, guint state, gpointer data)
 {
-	file_util_delete_multiple_review_back(NULL, data);
+	if (button == 1)
+		{
+		file_util_delete_multiple_review_next(NULL, data);
+		}
+	else if (button == 2 || button == 3)
+		{
+		file_util_delete_multiple_review_back(NULL, data);
+		}
 }
 
-static void file_util_delete_multiple_review_btn_next(ImageWindow *imd, guint32 time,
-						      gdouble x, gdouble y, guint state, gpointer data)
+static void file_util_delete_multiple_review_scroll_cb(ImageWindow *imd, GdkScrollDirection direction, guint32 time,
+						       gdouble x, gdouble y, guint state, gpointer data)
 {
-	file_util_delete_multiple_review_next(NULL, data);
+	if (direction == GDK_SCROLL_UP)
+		{
+		file_util_delete_multiple_review_back(NULL, data);
+		}
+	else if (direction == GDK_SCROLL_DOWN)
+		{
+		file_util_delete_multiple_review_next(NULL, data);
+		}
 }
 
 static void file_util_delete_multiple(GList *source_list)
@@ -1328,10 +1342,8 @@ static void file_util_delete_multiple(GList *source_list)
 
 		generic_dialog_image_add(gd, NULL, NULL);
 		imd = g_object_get_data(G_OBJECT(gd->dialog), "img_image");
-		image_set_button_func(imd, 1, file_util_delete_multiple_review_btn_next, gd);
-		image_set_button_func(imd, 5, file_util_delete_multiple_review_btn_next, gd);
-		image_set_button_func(imd, 2, file_util_delete_multiple_review_btn_back, gd);
-		image_set_button_func(imd, 4, file_util_delete_multiple_review_btn_back, gd);
+		image_set_button_func(imd, file_util_delete_multiple_review_button_cb, gd);
+		image_set_scroll_func(imd, file_util_delete_multiple_review_scroll_cb, gd);
 
 		hbox = gtk_hbox_new(FALSE, 5);
 		gtk_box_pack_start(GTK_BOX(gd->vbox), hbox, FALSE, FALSE, 0);
