@@ -653,7 +653,7 @@ static gint vdlist_auto_scroll_idle_cb(gpointer data)
 
 		window = vdl->listview->window;
 		gdk_window_get_pointer(window, &x, &y, NULL);
-		gdk_window_get_size(window, &w, &h);
+		gdk_drawable_get_size(window, &w, &h);
 		if (x >= 0 && x < w && y >= 0 && y < h)
 			{
 			vdlist_drop_update(vdl, x, y);
