@@ -711,7 +711,7 @@ static gint vdtree_auto_scroll_idle_cb(gpointer data)
 
 		window = vdt->treeview->window;
 		gdk_window_get_pointer(window, &x, &y, NULL);
-		gdk_window_get_size(window, &w, &h);
+		gdk_drawable_get_size(window, &w, &h);
 		if (x >= 0 && x < w && y >= 0 && y < h)
 			{
 			vdtree_drop_update(vdt, x, y);
@@ -1309,7 +1309,7 @@ gint vdtree_set_path(ViewDirTree *vdt, const gchar *path)
 		GtkTreeModel *store;
 		GtkTreePath *tpath;
 
-		tree_view_row_make_visible(GTK_TREE_VIEW(vdt->treeview), &iter);
+		tree_view_row_make_visible(GTK_TREE_VIEW(vdt->treeview), &iter, TRUE);
 
 		store = gtk_tree_view_get_model(GTK_TREE_VIEW(vdt->treeview));
 		tpath = gtk_tree_model_get_path(store, &iter);
