@@ -1082,7 +1082,7 @@ static void vficon_set_focus(ViewFileIcon *vfi, FileData *fd)
 		GtkTreeViewColumn *column;
 		GtkTreeModel *store;
 
-		tree_view_row_make_visible(GTK_TREE_VIEW(vfi->listview), &iter);
+		tree_view_row_make_visible(GTK_TREE_VIEW(vfi->listview), &iter, FALSE);
 
 		store = gtk_tree_view_get_model(GTK_TREE_VIEW(vfi->listview));
 		tpath = gtk_tree_model_get_path(store, &iter);
@@ -1575,7 +1575,7 @@ static void vficon_populate(ViewFileIcon *vfi, gint resize, gint keep_position)
 		if (g_list_find(list, visible_fd) == NULL &&
 		    vficon_find_iter(vfi, visible_fd, &iter, NULL))
 			{
-			tree_view_row_make_visible(GTK_TREE_VIEW(vfi->listview), &iter);
+			tree_view_row_make_visible(GTK_TREE_VIEW(vfi->listview), &iter, FALSE);
 			}
 		}
 
@@ -1665,6 +1665,8 @@ static void vficon_sync(ViewFileIcon *vfi)
 		g_list_free(list);
 		}
 
+	vfi->rows = r;
+
 	vficon_update_focus(vfi);
 }
 
