@@ -93,6 +93,18 @@ static void vflist_color_set(ViewFileList *vfl, FileData *fd, gint color_set)
 	gtk_list_store_set(GTK_LIST_STORE(store), &iter, FILE_COLUMN_COLOR, color_set, -1);
 }
 
+static void vflist_move_cursor(ViewFileList *vfl, GtkTreeIter *iter)
+{
+	GtkTreeModel *store;
+	GtkTreePath *tpath;
+
+	store = gtk_tree_view_get_model(GTK_TREE_VIEW(vfl->listview));
+
+	tpath = gtk_tree_model_get_path(store, iter);
+	gtk_tree_view_set_cursor(GTK_TREE_VIEW(vfl->listview), tpath, NULL, FALSE);
+	gtk_tree_path_free(tpath);
+}
+
 /*
  *-----------------------------------------------------------------------------
  * dnd
@@ -553,6 +565,7 @@ static gint vflist_press_cb(GtkWidget *widget, GdkEventButton *bevent, gpointer
 	    !(bevent->state & GDK_CONTROL_MASK ) &&
 	    vflist_row_is_selected(vfl, fd))
 		{
+		gtk_widget_grab_focus(widget);
 		return TRUE;
 		}
 
@@ -624,6 +637,7 @@ static gint vflist_release_cb(GtkWidget *widget, GdkEventButton *bevent, gpointe
 		selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(widget));
 		gtk_tree_selection_unselect_all(selection);
 		gtk_tree_selection_select_iter(selection, &iter);
+		vflist_move_cursor(vfl, &iter);
 		return TRUE;
 		}
 
@@ -1160,6 +1174,8 @@ void vflist_select_by_path(ViewFileList *vfl, const gchar *path)
 	if (vflist_row_by_path(vfl, path, &fd) < 0) return;
 	if (vflist_find_row(vfl, fd, &iter) < 0) return;
 
+	tree_view_row_make_visible(GTK_TREE_VIEW(vfl->listview), &iter, TRUE);
+
 	if (!vflist_row_is_selected(vfl, fd))
 		{
 		GtkTreeSelection *selection;
@@ -1169,14 +1185,13 @@ void vflist_select_by_path(ViewFileList *vfl, const gchar *path)
 		selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(vfl->listview));
 		gtk_tree_selection_unselect_all(selection);
 		gtk_tree_selection_select_iter(selection, &iter);
+		vflist_move_cursor(vfl, &iter);
 
 		store = gtk_tree_view_get_model(GTK_TREE_VIEW(vfl->listview));
 		tpath = gtk_tree_model_get_path(store, &iter);
 		gtk_tree_view_set_cursor(GTK_TREE_VIEW(vfl->listview), tpath, NULL, FALSE);
 		gtk_tree_path_free(tpath);
 		}
-
-	tree_view_row_make_visible(GTK_TREE_VIEW(vfl->listview), &iter);
 }
 
 /*
@@ -1749,6 +1764,7 @@ gint vflist_maint_removed(ViewFileList *vfl, const gchar *path, GList *ignore_li
 
 				selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(vfl->listview));
 				gtk_tree_selection_select_iter(selection, &iter);
+				vflist_move_cursor(vfl, &iter);
 				}
 			}
 		}
